from glob import glob
from os import path


class VoiceDirectoryChecker:
    def __init__(self) -> None:
        pass

    @staticmethod
    def check(dir_: str) -> bool:
        def count_wavs(dir_: str) -> int:
            return len(glob(path.join(dir_, "**/*.wav"), recursive=True))

        if not path.isdir(dir_):
            return False
        else:
            f = [
                path.isdir(i)
                and path.isdir(i + "/ボーイッシュ")
                and path.isdir(i + "/ロリ")
                and path.isdir(i + "/低音お姉さん")
                for i in (
                    dir_,
                    dir_ + "/【左耳】2021.0612",
                )
            ]
            return all(f) and count_wavs(dir_) == 138



from glob import glob
from multiprocessing import Process
from os import path
from random import randint, sample, shuffle
from typing import List, Optional

import simpleaudio as sa  # type: ignore[import]

from randomnipple.NippleOptionDict import NippleOptionDict


class RandomPlayer:
    def __init__(self, vals: NippleOptionDict) -> None:
        def get_one(options: List[str]) -> str:
            return [i for i in options if i in self.vals][0]

        self.vals = vals
        self.ear = get_one(["left", "right"])
        self.actor = get_one(["loli", "boyish", "low"])
        self.serif = get_one(["serif_enable", "serif_disable"])
        self.finish = get_one(["wet", "dry"])
        self.first = get_one(["first_enable", "first_disable"])
        self.repeat = vals["repeat"]
        self.dir = self.__set_dir()
        self.thread = Process(target=self.__play_loop)
        self.play_obj: Optional[sa.shiny.PlayObject] = None

    def __set_dir(self) -> str:
        base = self.vals["dir"]
        d = base if self.ear == "left" else path.join(base, "【左耳】2021.0612")
        t = (
            "ロリ"
            if self.actor == "loli"
            else "ボーイッシュ"
            if self.actor == "boyish"
            else "低音お姉さん"
        )
        return path.join(d, t)

    def play(self) -> None:
        self.thread.start()

    def __play_loop(self) -> None:
        while True:
            self.__play()

    def __play(self) -> None:
        self.__play_first()
        if self.serif == "serif_enable":
            loop_tracks = sorted(glob(path.join(self.dir, "ループ音声", "*.wav")))[1:-4]
            serif_tracks = sorted(glob(path.join(self.dir, "ループ音声", "*.wav")))[-4:]
            shuffle(serif_tracks)
            for s_track in serif_tracks:
                for track in sample(loop_tracks, randint(1, 8)):
                    self.play_track(track)
                else:
                    self.play_track(s_track)

        else:
            loop_tracks = glob(path.join(self.dir, "オノマトペオンリー", "*.wav"))
            shuffle(loop_tracks)
            for track in loop_tracks:
                self.__play_times(track)

    def play_track(self, track_path: str) -> None:
        if path.isfile(track_path):
            wav_obj = sa.WaveObject.from_wave_file(track_path)
            self.play_obj = wav_obj.play()
            if self.play_obj is not None:
                self.play_obj.wait_done()

    def play_final(self) -> None:
        self.thread = Process(target=self.__play_final)
        self.thread.start()

    def __play_times(self, track: str) -> None:
        for _ in range(self.repeat):
            self.play_track(track)

    def __play_first(self) -> None:
        if self.first == "first_enable":
            first_track = path.join(self.dir, "ループ音声", "01.この音声からランダム再生を初めてください.wav")
            self.play_track(first_track)

    def __play_final(self) -> None:
        final_track = path.join(
            self.dir,
            "絶頂音声",
            ("14.射精用トラック.wav" if self.finish == "wet" else "15.乳首ドライ絶頂用トラック.wav"),
        )
        self.play_track(final_track)

    def stop(self) -> None:
        if self.play_obj is not None and self.play_obj.is_playing():
            self.play_obj.stop()
        if self.thread.is_alive():
            self.thread.terminate()



#!/usr/bin/env python

from randomnipple.MainWindow import MainWindow


def main() -> None:
    m = MainWindow()
    m.launch()


if __name__ == "__main__":
    try:
        main()
    except KeyboardInterrupt:
        pass



from typing import Any, List

import PySimpleGUI as sg  # type: ignore[import]

from randomnipple import __version__
from randomnipple.NippleOptionDict import NippleOptionDict
from randomnipple.RandomPlayer import RandomPlayer
from randomnipple.VoiceDirectoryChecker import VoiceDirectoryChecker

sg.theme("DarkAmber")


class MainWindow:
    def __init__(self) -> None:
        self.layout = self.__layout()

        self.window = sg.Window(
            "Random Nipple Player v" + __version__, self.layout, keep_on_top=True
        )

    def __layout(self) -> List[List[Any]]:
        select_folder = [
            [
                sg.InputText(key="dir", size=(50, 1)),
                sg.FolderBrowse(target="dir", key="browse"),
            ]
        ]

        ear = [
            [
                sg.Radio("右耳　　", key="left", group_id="ear", default=True),
                sg.Radio("左耳　", key="right", group_id="ear"),
            ]
        ]

        actor = [
            [
                sg.Radio("ロリ", key="loli", group_id="actor", default=True),
                sg.Radio("ボーイッシュ", key="boyish", group_id="actor"),
                sg.Radio("低音お姉さん", key="low", group_id="actor"),
            ]
        ]

        first = [
            [
                sg.Radio("有り　　", key="first_enable", group_id="first", default=True),
                sg.Radio("無し　", key="first_disable", group_id="first"),
            ]
        ]

        serif = [
            [
                sg.Radio("有り", key="serif_enable", group_id="serif", default=True),
                sg.Radio("無し(オノマトペモード)　　　　 ", key="serif_disable", group_id="serif"),
            ]
        ]

        finish = [
            [
                sg.Radio("ウェット", key="wet", group_id="finish", default=True),
                sg.Radio("ドライ", key="dry", group_id="finish"),
            ]
        ]

        spin = [
            [
                sg.Spin(
                    [_ for _ in range(1, 100)],
                    initial_value=1,
                    key="repeat",
                    size=(5, 1),
                ),
                sg.Text("回　　　　　　　　　　　　　　　　"),
            ]
        ]

        return [
            [sg.Frame("フォルダ選択", [[sg.Column(select_folder, size=(460, 40))]])],
            [sg.Frame("方向", ear), sg.Frame("ボイス", actor)],
            [sg.Frame("挨拶", first), sg.Frame("ランダムセリフ", serif)],
            [sg.Frame("フィニッシュ", finish), sg.Frame("同一トラックリピート回数(オノマトペモード時)", spin)],
            [
                sg.Button("Play!", key="play", button_color="green"),
                sg.Button("Close", key="close", button_color="red"),
            ],
        ]

    def launch(self) -> None:
        signal = True
        while signal:
            signal = self.__do()
            self.__toggle_button(False)
        else:
            self.window.close()

    def __toggle_button(self, flag: bool) -> None:
        for i in ("browse", "play", "close"):
            self.window[i].update(disabled=flag)

    def __do(self) -> bool:
        vals: NippleOptionDict
        evt, vals = self.window.read()
        # print(vals)  # debug
        if evt == "play":
            self.__toggle_button(True)
            if not VoiceDirectoryChecker.check(vals["dir"]):
                sg.PopupError(
                    "指定したフォルダは有効ではありません。\n" "指定したフォルダ: " + vals["dir"], keep_on_top=True
                )
                return True
            r = RandomPlayer(vals)
            r.play()
            rtn = sg.popup(
                "再生中: 焦らしパート",
                title="再生中...",
                custom_text=("Finish!", "Cancel"),
                keep_on_top=True,
            )
            if rtn == "Finish!":
                r.stop()
                r.play_final()
                rtn = sg.popup(
                    "再生中: フィニッシュパート",
                    title="再生中...",
                    custom_text="Cancel",
                    keep_on_top=True,
                )
            r.stop()
            return True
        else:
            return False



from typing import TypedDict


class NippleOptionDict(TypedDict):
    boyish: bool
    dir: str
    dry: bool
    first_disable: bool
    first_enable: bool
    left: bool
    loli: bool
    low: bool
    repeat: int
    right: bool
    serif_disable: bool
    serif_enable: bool
    wet: bool
